@@ -1,19 +1,23 @@
-# Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2 of the
-# License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms, as
+# designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
+# This program is distributed in the hope that it will be useful,  but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+# the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-# 02110-1301  USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 import re
 import string
@@ -179,6 +183,7 @@ class MSSQLMigration(GenericMigration):
 
                     # Only timestamp supports CURRENT_TIMESTAMP, so force the target type to it
                     target_column.simpleType = find_object_with_name(state.targetCatalog.simpleDatatypes, 'TIMESTAMP')
+                    target_column.length = -1
                     state.addMigrationLogEntry(0, source_column, target_column, 
                               'Default value is %s, so type was changed from %s to TIMESTAMP' % (default_value, source_datatype))
 
@@ -208,10 +213,15 @@ class MSSQLMigration(GenericMigration):
             target_column.flags.extend(source_column.userType.flags)
 
         if source_type:
+            target_version = Version.fromgrt(targetCatalog.version)
             # Decide which mysql datatype corresponds to the column datatype:
             source_datatype = source_type.name.upper()
+            grt.log_debug3("Migration", "Migrating source column '%s' - type: %s, length: %s\n" % (source_column.name, source_datatype,source_column.length))
             # string data types:
             target_datatype = ''
+            #NCHAR and NVARCHAR in Microsoft SQL Server is always encoded as UCS-2 (UTF-16)
+            if source_datatype in ['NCHAR', 'NVARCHAR'] and target_version.is_supported_mysql_version_at_least(5,5,0):
+                target_column.characterSetName = 'utf8mb4'
             if source_datatype in ['VARCHAR', 'NVARCHAR']:
                 if source_column.length == -1:  # VARCHAR(MAX) or NVARCHAR(MAX)
                     target_datatype = 'LONGTEXT'  #TODO: Give the user the choice for this target datatype
@@ -272,24 +282,13 @@ class MSSQLMigration(GenericMigration):
                 target_column.scale = source_column.simpleType.numericScale
             # binary datatypes:
             elif source_datatype == 'IMAGE':
-                if source_column.length == -1:  # VARBINARY(MAX)
-                   target_datatype = 'LONGBLOB'  #TODO: Give the user the choice for this target datatype
-                elif 0 <= source_column.length < 256:
-                    if source_datatype == 'IMAGE':
-                        target_datatype = 'TINYBLOB'
-                    else:
-                        target_datatype = source_datatype
-                elif 0 <= source_column.length < 65536:
-                    target_datatype = 'MEDIUMBLOB'
-                else:
-                    target_datatype = 'LONGBLOB'
+                target_datatype = 'LONGBLOB'
             elif source_datatype == 'VARBINARY' and source_column.length == -1:  # VARBINARY(MAX):
                 target_datatype = 'LONGBLOB'
             # datetime datatypes:
             elif source_datatype in ['DATETIME', 'SMALLDATETIME', 'DATETIME2', 'DATETIMEOFFSET']:
                 target_datatype = 'DATETIME'
                 target_column.precision = -1
-                target_version = Version.fromgrt(targetCatalog.version)
                 if target_version.is_supported_mysql_version_at_least(5,6,4) and source_datatype != 'SMALLDATETIME':
                     target_column.precision = source_column.precision if source_column.precision < 7 else 6
             # timestamp datatypes
@@ -303,7 +302,6 @@ class MSSQLMigration(GenericMigration):
             elif source_datatype == 'TIME':
                 target_datatype = 'TIME'
                 target_column.precision = -1
-                target_version = Version.fromgrt(targetCatalog.version)
                 if target_version.is_supported_mysql_version_at_least(5,6,4):
                     target_column.precision = source_column.precision if source_column.precision < 7 else 6
             elif source_datatype == 'BIT':
@@ -333,7 +331,7 @@ class MSSQLMigration(GenericMigration):
             if mysql_simpleTypes.has_key(target_datatype):
                 target_column.simpleType = mysql_simpleTypes[target_datatype]
             else:
-                grt.log_warning("MSSQL migrateTableColumnsToMySQL", "Can't find datatype %s for type %s\n" % (target_datatype, source_datatype))
+                grt.log_warning("Migration", "MSSQL migrateTableColumnsToMySQL", "Can't find datatype %s for type %s\n" % (target_datatype, source_datatype))
                 state.addMigrationLogEntry(2, source_column, target_column, 
                     'Could not migrate column "%s" in "%s": Unknown datatype "%s"' % (target_column.name, source_column.owner.name, source_datatype) )
                 return False
