@@ -1,20 +1,23 @@
-# Copyright (c) 2013, 2016, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2 of the
-# License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms, as
+# designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
+# This program is distributed in the hope that it will be useful,  but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+# the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-# 02110-1301  USA
-
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 import mforms
 import grt
@@ -25,9 +28,10 @@ from sqlide_catalogman_ext import MaintenanceResultForm
 from sqlide_catalogman_ext import ObjectManager
 from sqlide_catalogman_ext import TriggerManager
 from sqlide_catalogman_ext import ColumnManager
-from mforms import IconStringColumnType, StringColumnType, IntegerColumnType
+from mforms import IconStringColumnType, StringColumnType, LongIntegerColumnType
 from wb_admin_utils import make_panel_header
 from workbench.utils import human_size, Version
+from wb_common import to_unicode
 
 def make_title(t):
     l = mforms.newLabel(t)
@@ -41,7 +45,7 @@ def show_table_inspector(editor, selection, page = None):
         dpoint = mforms.fromgrt(editor.dockingPoint)
         dpoint.dock_view(tinspect, "", 0)
         dpoint.select_view(tinspect)
-        tinspect.set_title("%s.%s" % (schema, table))
+        tinspect.set_title(u"%s.%s" % (to_unicode(schema), to_unicode(table)))
         if page is not None:
             tinspect.switch_to_page(page)
 
@@ -80,6 +84,7 @@ class TableInfoPanel(mforms.Box):
         self.set_release_on_add()
         
         self.editor = editor
+        self.target_version = Version.fromgrt(self.editor.serverVersion)
         
         self.table = mforms.newTable()
         offset = 2 if self.i_s_innodb_available() else 0
@@ -237,7 +242,9 @@ class TableInfoPanel(mforms.Box):
         self.refresh()
         
     def i_s_innodb_available(self):
-        return self.editor.serverVersion.majorNumber == 5 and self.editor.serverVersion.minorNumber == 6
+          return  self.target_version.is_supported_mysql_version_at_least(5, 6)
+
+        
 
     def refresh(self):
         try:
@@ -285,17 +292,25 @@ class TableInfoPanel(mforms.Box):
                 self.column_count.set_text(rset.stringFieldValueByName("column_count"));
             
         if self.i_s_innodb_available():
+            if self.target_version.is_supported_mysql_version_at_least(8, 0):
+                query = "SELECT @@datadir datadir,sd.path FROM information_schema.INNODB_TABLES st JOIN information_schema.innodb_datafiles sd USING(space) WHERE st.name = '%s/%s'" % (self._schema, self._table)
+            else:
+                query = "SELECT @@datadir datadir,st.FILE_FORMAT,sd.path FROM information_schema.INNODB_SYS_TABLES st JOIN information_schema.innodb_sys_datafiles sd USING(space) WHERE st.name = '%s/%s'" % (self._schema, self._table)
             try:
-                rset = self.editor.executeManagementQuery("SELECT @@datadir datadir,st.FILE_FORMAT,sd.path FROM information_schema.INNODB_SYS_TABLES st JOIN information_schema.innodb_sys_datafiles sd USING(space) WHERE st.name = '%s/%s'" % (self._schema, self._table), 0)
+                rset = self.editor.executeManagementQuery(query, 0)
             except grt.DBError, e:
-                log_error("SELECT @@datadir datadir,st.FILE_FORMAT,sd.path FROM information_schema.INNODB_SYS_TABLES st JOIN information_schema.innodb_sys_datafiles sd USING(space) WHERE st.name = '%s/%s'': %s\n" % (self._schema, self._table, e))
+                log_error("%s': %s\n" % (query, e))
                 rset = None
-
             if rset:
                 ok = rset.goToFirstRow()
                 if ok:
-                    self.file_format.set_text(rset.stringFieldValueByName("FILE_FORMAT"))
-                    data_path = rset.stringFieldValueByName("path")
+                    if self.target_version.is_supported_mysql_version_at_least(8, 0):
+                            self.file_format.set_text("")
+                            data_path = rset.stringFieldValueByName("PATH")
+                    else:
+                            self.file_format.set_text(rset.stringFieldValueByName("FILE_FORMAT"))
+                            data_path = rset.stringFieldValueByName("path")
+
                     if data_path[:1] == ".":
                         self.data_path.set_text("%s%s" %(rset.stringFieldValueByName("datadir"), data_path[2:]))
                     else:
@@ -558,12 +573,15 @@ class TableIndexInfoPanel(mforms.Box):
 
         table.add(make_title("Indexes in Table"), 0, 1, 0, 1, mforms.HFillFlag)
         self.index_list = mforms.newTreeView(mforms.TreeFlatList|mforms.TreeAltRowColors|mforms.TreeShowColumnLines)
+        if self.target_version.is_supported_mysql_version_at_least(8, 0, 0):
+            self.index_list.add_column(mforms.CheckColumnType, "Visible", 50, True)
         self.index_list.add_column(mforms.IconStringColumnType, "Key", 140, False)
         self.index_list.add_column(mforms.StringColumnType, "Type", 80, False)
         self.index_list.add_column(mforms.StringColumnType, "Unique", 40, False)
         self.index_list.add_column(mforms.StringColumnType, "Columns", 200, False)
         self.index_list.end_columns()
         self.index_list.add_changed_callback(self.index_selected)
+        self.index_list.set_cell_edited_callback(self.edit_index)
         self.index_list.set_size(450, -1)
         table.add(self.index_list, 0, 1, 1, 2, mforms.HFillFlag|mforms.VFillFlag)
 
@@ -652,11 +670,10 @@ class TableIndexInfoPanel(mforms.Box):
         hbox.add_end(self.create_index, False, True)
         self.add(hbox, False, True)
 
-
     def do_drop_index(self):
         node = self.index_list.get_selected_node()
         if node:
-            index = node.get_string(0)
+            index = node.get_string(1) if self.target_version.is_supported_mysql_version_at_least(8, 0, 0) else node.get_string(0)
             if mforms.Utilities.show_message("Drop Index", "Drop index `%s` from table `%s`.`%s`?" % (index, self._schema, self._table), "Drop", "Cancel", "") == mforms.ResultOk:
                 try:
                     self.editor.executeManagementCommand("DROP INDEX `%s` ON `%s`.`%s`" % (index, self._schema, self._table), 1)
@@ -664,7 +681,6 @@ class TableIndexInfoPanel(mforms.Box):
                 except grt.DBError, e:
                     mforms.Utilities.show_error("Drop Index", "Error dropping index.\n%s" % e.args[0], "OK", "", "")
 
-
     def do_create_index(self):
         cols = []
         for node in self.column_list.get_selection():
@@ -676,6 +692,14 @@ class TableIndexInfoPanel(mforms.Box):
         else:
             mforms.Utilities.show_warning("Create Index","You have to select at least one column.\n", "OK", "", "")
 
+    def edit_index(self, node, column, value):
+        if self.target_version.is_supported_mysql_version_at_least(8, 0, 0):
+            index = node.get_string(1)
+            sql = "ALTER TABLE `%s`.`%s` ALTER INDEX `%s` %s" % (self._schema, self._table, index, "INVISIBLE" if node.get_bool(0) is True else "VISIBLE")
+            if mforms.Utilities.show_message("Alter Index", "Do you really want to %s ?" % sql, "Yes", "Cancel", "") == mforms.ResultOk:
+                self.editor.executeManagementCommand(sql, 1)
+                node.set_bool(0, False if node.get_bool(0) is True else True)
+
     def index_selected(self):
         node = self.index_list.get_selected_node()
         if node:
@@ -739,22 +763,34 @@ class TableIndexInfoPanel(mforms.Box):
                 curname = None
                 itype = None
                 non_unique = None
+                is_visible = True
                 columns = []
                 while ok:
                     name = rset.stringFieldValue(2)
                     if name != curname:
                         if columns:
                             node = self.index_list.add_node()
-                            node.set_icon_path(0, index_icon)
-                            node.set_string(0, curname)
-                            node.set_string(1, itype)
-                            node.set_string(2, "YES" if non_unique != "1" else "NO")
-                            node.set_string(3, ", ".join([c[1] for c in columns]))
+                            if self.target_version.is_supported_mysql_version_at_least(8, 0, 0):
+                                node.set_bool(0, is_visible)
+                                node.set_icon_path(1, index_icon)
+                                node.set_string(1, curname)
+                                node.set_string(2, itype)
+                                node.set_string(3, "YES" if non_unique != "1" else "NO")
+                                node.set_string(4, ", ".join([c[1] for c in columns]))
+                            else:
+                                node.set_icon_path(0, index_icon)
+                                node.set_string(0, curname)
+                                node.set_string(1, itype)
+                                node.set_string(2, "YES" if non_unique != "1" else "NO")
+                                node.set_string(3, ", ".join([c[1] for c in columns]))
                         columns = []
                         self.index_info.append(([rset.stringFieldValue(i) for i in index_rs_columns], columns))
                         curname = name
                         itype = rset.stringFieldValue(10)
                         non_unique = rset.stringFieldValue(1)
+                        is_visible = True
+                        if self.target_version.is_supported_mysql_version_at_least(8, 0, 0):
+                            is_visible = rset.stringFieldValue(13) == "YES"
 
                     cname = rset.stringFieldValue(4)
                     if cname not in column_to_index:
@@ -763,13 +799,22 @@ class TableIndexInfoPanel(mforms.Box):
                         column_to_index[cname].append(name)
                     columns.append([rset.stringFieldValue(i) for i in column_rs_columns])
                     ok = rset.nextRow()
+
                 if columns:
                     node = self.index_list.add_node()
-                    node.set_icon_path(0, index_icon)
-                    node.set_string(0, curname)
-                    node.set_string(1, itype)
-                    node.set_string(2, "YES" if non_unique != "1" else "NO")
-                    node.set_string(3, ", ".join([c[1] for c in columns]))
+                    if self.target_version.is_supported_mysql_version_at_least(8, 0, 0):
+                        node.set_bool(0, is_visible)
+                        node.set_icon_path(1, index_icon)
+                        node.set_string(1, curname)
+                        node.set_string(2, itype)
+                        node.set_string(3, "YES" if non_unique != "1" else "NO")
+                        node.set_string(4, ", ".join([c[1] for c in columns]))
+                    else:
+                        node.set_icon_path(0, index_icon)
+                        node.set_string(0, curname)
+                        node.set_string(1, itype)
+                        node.set_string(2, "YES" if non_unique != "1" else "NO")
+                        node.set_string(3, ", ".join([c[1] for c in columns]))
             try:
                 rset = self.editor.executeManagementQuery("SHOW COLUMNS FROM `%s`.`%s`" % (schema, table), 0)
             except grt.DBError, e:
@@ -878,18 +923,18 @@ class PartitionManager(TableManDefs, ObjectManager):
     icon_column = 0
     columns = [("partition_name", IconStringColumnType, "Name", 200, None),
                ("subpartition_name", StringColumnType, "Subpartiton Name", 100, None),
-               ("partition_ordinal_position", IntegerColumnType, "Ordinal Pos", 100, None),
-               ("subpartition_ordinal_position", IntegerColumnType, "Subpartiton Ordinal Pos", 100, None),
+               ("partition_ordinal_position", LongIntegerColumnType, "Ordinal Pos", 100, None),
+               ("subpartition_ordinal_position", LongIntegerColumnType, "Subpartiton Ordinal Pos", 100, None),
                ("partition_method", StringColumnType, "Partition Method", 100, None),
                ("subpartition_method", StringColumnType, "Subpartition Method", 100, None),
                ("partition_expression", StringColumnType, "Partition expression", 100, None),
                ("subpartition_expression", StringColumnType, "Subpartition expression", 100, None),
                ("partition_description", StringColumnType, "Partition description", 100, None),
-               ("table_rows", IntegerColumnType, "Table rows", 100, None),
-               ("avg_row_length", IntegerColumnType, "AVG row length", 100, None),
-               ("data_length", IntegerColumnType, "Data length", 100, None),
-               ("max_data_length", IntegerColumnType, "max data length", 100, None),
-               ("index_length", IntegerColumnType, "Index length", 100, None),
+               ("table_rows", LongIntegerColumnType, "Table rows", 100, None),
+               ("avg_row_length", LongIntegerColumnType, "AVG row length", 100, None),
+               ("data_length", LongIntegerColumnType, "Data length", 100, None),
+               ("max_data_length", LongIntegerColumnType, "max data length", 100, None),
+               ("index_length", LongIntegerColumnType, "Index length", 100, None),
                ("create_time", StringColumnType, "Create time", 150, None),
                ("update_time", StringColumnType, "Update time", 150, None),
                ("check_time", StringColumnType, "Check time", 150, None)]
@@ -1134,11 +1179,12 @@ class TableInspector(mforms.AppView):
 
     def show_table(self, schema, table):
         for tab in self.pages:
-#             try:
-            tab.show_table(schema, table)
-#             except:
-#                 print "some error"
-#                 continue # If user will launch table inspector on Information or Performance Schema, there can be some problems, let's catch those.
+            try:
+                tab.show_table(schema, table)
+            except Exception:
+                import traceback
+                log_error("Error initializing tab %s: %s\n" % (tab.node_name, traceback.format_exc()))
+                continue # If user will launch table inspector on Information or Performance Schema, there can be some problems, let's catch those.
         #After each tab is loaded we can make a ptr to the get_table_engine method
         self.pages[self.tab_list["indexes"]].get_table_engine = self.pages[self.tab_list["informations"]].get_table_engine
 
