@@ -1,23 +1,27 @@
-# Copyright (c) 2007, 2016, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2 of the
-# License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms, as
+# designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
+# This program is distributed in the hope that it will be useful,  but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+# the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-# 02110-1301  USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 from workbench.db_utils import QueryError, escape_sql_string
 from workbench.utils import Version
-from wb_common import PermissionDeniedError
+from wb_common import PermissionDeniedError, to_unicode
 
 import mforms
 from workbench.log import log_error
@@ -36,67 +40,69 @@ ZOMBIE_COLUMN_PRIVS_QUERY = "SELECT c.User, c.Host, c.Db, c.Table_name, c.Column
 ZOMBIE_PROCS_PRIVS_QUERY = "SELECT p.User, p.Host, p.Db, p.Routine_name, p.Routine_type FROM mysql.procs_priv AS p LEFT JOIN mysql.user AS u ON p.User = u.user AND p.Host = u.Host WHERE u.User IS NULL"
 
 
-GET_ACCOUNT_QUERY = "SELECT * FROM mysql.user WHERE User='%(user)s' AND Host='%(host)s' ORDER BY User, Host"
+GET_ACCOUNT_QUERY = u"SELECT * FROM mysql.user WHERE User='%(user)s' AND Host='%(host)s' ORDER BY User, Host"
 
-GET_ACCOUNT_SCHEMA_PRIVS_QUERY = "SELECT * FROM mysql.db WHERE User='%(user)s' AND Host='%(host)s' ORDER BY Db"
+GET_ACCOUNT_SCHEMA_PRIVS_QUERY = u"SELECT * FROM mysql.db WHERE User='%(user)s' AND Host='%(host)s' ORDER BY Db"
 
-GET_ACCOUNT_MYSQL_TABLE_PRIVS_QUERY = "SELECT * FROM mysql.tables_priv WHERE Host='%(host)s' AND User='%(user)s' AND Db='mysql'"
+GET_ACCOUNT_MYSQL_TABLE_PRIVS_QUERY = u"SELECT * FROM mysql.tables_priv WHERE Host='%(host)s' AND User='%(user)s' AND Db='mysql'"
 #GET_ACCOUNT_IS_TABLE_PRIVS_QUERY = "SELECT * FROM mysql.tables_priv WHERE Host='%(host)s' AND User='%(user)s' AND Db='information_schema'"
 
-CREATE_USER_QUERY = "CREATE USER '%(user)s'@'%(host)s' IDENTIFIED BY '%(password)s'"
-CREATE_USER_QUERY_PLUGIN_AUTH_STRING = "CREATE USER '%(user)s' IDENTIFIED WITH '%(auth_plugin)s' AS '%(auth_string)s'"
-CREATE_USER_QUERY_PLUGIN = "CREATE USER '%(user)s' IDENTIFIED WITH '%(auth_plugin)s'"
+CREATE_USER_QUERY = u"CREATE USER '%(user)s'@'%(host)s' IDENTIFIED BY '%(password)s'"
+CREATE_USER_QUERY_PLUGIN_AUTH_STRING = u"CREATE USER '%(user)s' IDENTIFIED WITH '%(auth_plugin)s' AS '%(auth_string)s'"
+CREATE_USER_QUERY_PLUGIN = u"CREATE USER '%(user)s' IDENTIFIED WITH '%(auth_plugin)s'"
+CREATE_USER_QUERY_PLUGIN_AUTH_CACHING = u"CREATE USER '%(user)s' IDENTIFIED WITH '%(auth_plugin)s' BY '%(password)s'"
 
-GRANT_GLOBAL_PRIVILEGES_QUERY = "GRANT %(granted_privs)s ON *.* TO '%(user)s'@'%(host)s'"  # A WITH clause will be added if needed
-REVOKE_GLOBAL_PRIVILEGES_QUERY = "REVOKE %(revoked_privs)s ON *.* FROM '%(user)s'@'%(host)s'"
-GRANT_LIMITS_QUERY = "GRANT USAGE ON *.* TO '%(user)s'@'%(host)s' WITH %(limit)s"
-RENAME_USER_QUERY = "RENAME USER '%(old_user)s'@'%(old_host)s' TO '%(user)s'@'%(host)s'"
-CHANGE_PASSWORD_QUERY = "SET PASSWORD FOR '%(user)s'@'%(host)s' = PASSWORD('%(password)s')"
-BLANK_PASSWORD_QUERY = "SET PASSWORD FOR '%(user)s'@'%(host)s' = ''"
-CHANGE_PASSWORD_QUERY_576 = "ALTER USER '%(user)s'@'%(host)s' IDENTIFIED BY '%(password)s'"
-BLANK_PASSWORD_QUERY_576 = "ALTER USER '%(user)s'@'%(host)s' IDENTIFIED BY ''"
+ALTER_USER_RESOURCES = u"ALTER USER '%(user)s'@'%(host)s'" # A WITH clause will be added
+GRANT_GLOBAL_PRIVILEGES_QUERY = u"GRANT %(granted_privs)s ON *.* TO '%(user)s'@'%(host)s'"  # A WITH clause will be added if needed
+REVOKE_GLOBAL_PRIVILEGES_QUERY = u"REVOKE %(revoked_privs)s ON *.* FROM '%(user)s'@'%(host)s'"
+GRANT_LIMITS_QUERY = u"GRANT USAGE ON *.* TO '%(user)s'@'%(host)s' WITH %(limit)s"
+RENAME_USER_QUERY = u"RENAME USER '%(old_user)s'@'%(old_host)s' TO '%(user)s'@'%(host)s'"
+CHANGE_PASSWORD_QUERY = u"SET PASSWORD FOR '%(user)s'@'%(host)s' = PASSWORD('%(password)s')"
+BLANK_PASSWORD_QUERY = u"SET PASSWORD FOR '%(user)s'@'%(host)s' = ''"
+CHANGE_PASSWORD_QUERY_576 = u"ALTER USER '%(user)s'@'%(host)s' IDENTIFIED BY '%(password)s'"
+BLANK_PASSWORD_QUERY_576 = u"ALTER USER '%(user)s'@'%(host)s' IDENTIFIED BY ''"
 
-REVOKE_SCHEMA_PRIVILEGES_QUERY = "REVOKE %(revoked_privs)s ON `%(db)s`.* FROM '%(user)s'@'%(host)s'"
-GRANT_SCHEMA_PRIVILEGES_QUERY = "GRANT %(granted_privs)s ON `%(db)s`.* TO '%(user)s'@'%(host)s'"
+REVOKE_SCHEMA_PRIVILEGES_QUERY = u"REVOKE %(revoked_privs)s ON `%(db)s`.* FROM '%(user)s'@'%(host)s'"
+GRANT_SCHEMA_PRIVILEGES_QUERY = u"GRANT %(granted_privs)s ON `%(db)s`.* TO '%(user)s'@'%(host)s'"
 
-EXPIRE_PASSWORD = "ALTER USER '%(user)s'@'%(host)s' PASSWORD EXPIRE"
+EXPIRE_PASSWORD = u"ALTER USER '%(user)s'@'%(host)s' PASSWORD EXPIRE"
 FLUSH_PRIVILEGES = "FLUSH PRIVILEGES"
 
-REVOKE_ALL = "REVOKE ALL PRIVILEGES, GRANT OPTION FROM '%(user)s'@'%(host)s'"
-REMOVE_USER = "DROP USER '%(user)s'@'%(host)s'"
+REVOKE_ALL = u"REVOKE ALL PRIVILEGES, GRANT OPTION FROM '%(user)s'@'%(host)s'"
+REMOVE_USER = u"DROP USER '%(user)s'@'%(host)s'"
 
 # Map of user table's column name to privilege and its description.
 # It has the form { 'TableColumnPrivName': ('PrettyName', 'ADescriptionOfThePrivilege'), ... }
 PrivilegeInfo = {
-"Select_priv": ("SELECT", "The SELECT privilege enables you to select rows from tables in a database.\nSELECT statements require the SELECT privilege only if they actually retrieve rows from a table. Some SELECT statements do not access tables and can be executed without permission for any database"),
-"Insert_priv": ("INSERT", "The INSERT privilege enables you to be inserted into tables in a database.\nINSERT is also required for the ANALYZE TABLE, OPTIMIZE TABLE, and REPAIR TABLE table-maintenance statements."),
-"Update_priv": ("UPDATE", "The UPDATE privilege enables you to be updated in tables in a database."),
-"Delete_priv": ("DELETE", "The DELETE privilege enables you to be deleted from tables in a database."),
-"Create_priv": ("CREATE", "The CREATE privilege enables creation of new databases and tables."),
-"Drop_priv": ("DROP", """The DROP privilege enables you to drop (remove) existing databases, tables, and views.
+"Select_priv": ("SELECT", "The SELECT privilege enables querying data from tables and views.\nSELECT statements require the SELECT privilege only if they actually retrieve rows from a table. Some SELECT statements do not access tables and can be executed without permission for any database."),
+"Insert_priv": ("INSERT", "The INSERT privilege enables the insertion of data into tables.\nINSERT is also required for the ANALYZE TABLE, OPTIMIZE TABLE, and REPAIR TABLE table-maintenance statements."),
+"Update_priv": ("UPDATE", "The UPDATE privilege enables changing data in tables."),
+"Delete_priv": ("DELETE", "The DELETE privilege enables deletion of data from tables."),
+"Create_priv": ("CREATE", "The CREATE privilege enables creation of new databases, tables, views, and more."),
+"Drop_priv": ("DROP", """The DROP privilege enables you to drop (remove) existing databases, tables, views, and more.
 Beginning with MySQL 5.1.10, the DROP privilege is also required in order to use the statement ALTER TABLE ... DROP PARTITION on a partitioned table.
-Beginning with MySQL 5.1.16, the DROP privilege is required for TRUNCATE TABLE (before that, TRUNCATE TABLE requires the DELETE privilege).
-If you grant the DROP privilege for the mysql database to a user, that user can drop the database in which the MySQL access privileges are stored."""),
+Beginning with MySQL 5.1.16, the DROP privilege is required for TRUNCATE TABLE (before that, TRUNCATE TABLE required the DELETE privilege).
+Warning: if you grant the DROP privilege for the 'mysql' database to a user, that user can drop the database in which the MySQL access privileges are stored."""),
 "Reload_priv": ("RELOAD", "The RELOAD privilege enables use of the FLUSH statement.\nIt also enables mysqladmin commands that are equivalent to FLUSH operations:\nflush-hosts, flush-logs, flush-privileges, flush-status, flush-tables, flush-threads, refresh, and reload."),
 "Shutdown_priv": ("SHUTDOWN", "The SHUTDOWN privilege enables use of the mysqladmin shutdown command. There is no corresponding SQL statement."),
 "Process_priv": ("PROCESS", "The PROCESS privilege pertains to display of information about the threads executing within the server\n(that is, information about the statements being executed by sessions).\nThe privilege enables use of SHOW PROCESSLIST or mysqladmin processlist to see threads belonging to other accounts; you can always see your own threads."),
 "File_priv": ("FILE", """The FILE privilege gives you permission to read and write files on the server host using the
 LOAD DATA INFILE and SELECT ... INTO OUTFILE statements and the LOAD_FILE() function. A user who has the FILE privilege
 can read any file on the server host that is either world-readable or readable by the MySQL server.
-(This implies the user can read any file in any database directory, because the server can access any of those files.)
+This implies the user can read any file in any database directory, because the server can access any of those files.
 The FILE privilege also enables the user to create new files in any directory where the MySQL server has write access.
 As a security measure, the server will not overwrite existing files."""),
 "Grant_priv": ("GRANT OPTION", "The GRANT OPTION privilege enables you to give to other users or remove from other users those privileges that you yourself possess."),
 "References_priv": ("REFERENCES", "The REFERENCES privilege currently is unused."),
 "Index_priv": ("INDEX", "The INDEX privilege enables you to create or drop (remove) indexes.\nINDEX applies to existing tables. If you have the CREATE privilege for a table, you can include index definitions in the CREATE TABLE statement."),
-"Alter_priv": ("ALTER", "The ALTER privilege enables use of ALTER TABLE to change the structure of or rename tables.\n(ALTER TABLE also requires the INSERT and CREATE privileges.)"),
+"Alter_priv": ("ALTER", "The ALTER privilege enables use of ALTER TABLE to change the structure of or rename tables.\nALTER TABLE also requires the INSERT and CREATE privileges."),
 "Show_db_priv": ("SHOW DATABASES", """The SHOW DATABASES privilege enables the account to see database names by issuing the
 SHOW DATABASE statement. Accounts that do not have this privilege see only databases for which
 they have some privileges, and cannot use the statement at all if the server was started with
 the --skip-show-database option. Note that any global privilege is a privilege for the database."""),
 "Super_priv": ("SUPER", """The SUPER privilege enables an account to use CHANGE MASTER TO, KILL or
 mysqladmin kill to kill threads belonging to other accounts (you can always kill your own threads),
-PURGE BINARY LOGS, configuration changes via SET GLOBAL to modify global system variables,
+PURGE BINARY LOGS, configuration changes with SET GLOBAL to modify global system variables,
 the mysqladmin debug command, enabling or disabling logging, performing updates even if the read_only
 system variable is enabled, starting and stopping replication on slave servers, and allows you to
 connect (once) even if the connection limit controlled by the max_connections system variable is reached."""),
@@ -261,7 +267,7 @@ class AdminSecurity(object):
                 result = self.ctrl_be.exec_query("DESCRIBE mysql.db")
             except QueryError, e:
                 if e.error == 1142:
-                    raise PermissionDeniedError("Please make sure the used account has rights to the MySQL grant tables.\n%s" % e)
+                    raise PermissionDeniedError("Please make sure the account used has rights to the MySQL grant tables.\n%s" % e)
                 raise e
 
             if result is not None:
@@ -275,7 +281,7 @@ class AdminSecurity(object):
                 result = self.ctrl_be.exec_query("DESCRIBE mysql.user")
             except QueryError, e:
                 if e.error == 1142:
-                    raise PermissionDeniedError("Please make sure the used account has rights to the MySQL grant tables.\n%s" % e)
+                    raise PermissionDeniedError("Please make sure the account used has rights to the MySQL grant tables.\n%s" % e)
                 raise e
 
             if result is not None:
@@ -299,14 +305,14 @@ class AdminSecurity(object):
             result = self.ctrl_be.exec_query(LIST_SCHEMAS_QUERY)
         except QueryError, e:
             if e.error == 1142:
-                raise PermissionDeniedError("Please make sure the used account has rights to the MySQL grant tables.\n%s" % e)
+                raise PermissionDeniedError("Please make sure the account used has rights to the MySQL grant tables.\n%s" % e)
             raise e
         except Exception, e:
             raise Exception("Error querying privilege information: %s" % e)
 
         if result is not None:
             while result.nextRow():
-                name = result.stringByName("Database")
+                name = to_unicode(result.stringByName("Database"))
                 schema_names.append(name)
 
         schema_names.sort()
@@ -321,8 +327,8 @@ class AdminSecurity(object):
 
         if result:
             while result.nextRow():
-                user = result.stringByName("User")
-                host = result.stringByName("Host")
+                user = to_unicode(result.stringByName("User"))
+                host = to_unicode(result.stringByName("Host"))
                 accounts.append((user, host))
 
 
@@ -335,9 +341,9 @@ class AdminSecurity(object):
                 return []
             privs = []
             while result and result.nextRow():
-                user = result.stringByName("User")
-                host = result.stringByName("Host")
-                parts = [result.stringByName(f) for f in fields]
+                user = to_unicode(result.stringByName("User"))
+                host = to_unicode(result.stringByName("Host"))
+                parts = [to_unicode(result.stringByName(f)) for f in fields]
                 privs.append(((user, host), ".".join(parts)))
             return privs
 
@@ -375,6 +381,7 @@ class AdminSecurity(object):
 
     @property
     def account_names(self):
+        self._accounts.sort() 
         return self._accounts
 
 
@@ -429,8 +436,8 @@ class AdminSecurity(object):
             return name
 
         acct = AdminAccount(self)
-        acct.host = "%"
-        acct.username = unique_name('newuser', acct.host)
+        acct.host = u"%"
+        acct.username = to_unicode(unique_name(u'newuser', acct.host))
         self._account_info_cache[acct.username+"@"+acct.host] = acct
         self._accounts.append((acct.username, acct.host))
         return acct
@@ -537,7 +544,7 @@ class AdminUserDbPrivs(object):
                 if value == 'Y':
                     privs.add(priv)
 
-            schema = result.stringByName("Db")
+            schema = to_unicode(result.stringByName("Db"))
 
             self.entries.append(AdminUserDbPrivEntry(schema, privs))
 
@@ -755,7 +762,7 @@ class AdminAccount(object):
             if (self.username, self.host) in self._owner.account_names:
                 raise WBSecurityValidationError("The '%s' account already exists and cannot be saved." % (self.formatted_name()))
         elif not self.is_commited:
-            if self._owner.account_names.count((self.username, self.host)) > 1:
+            if (self.username, self.host) in self._owner.account_names:
                 raise WBSecurityValidationError("The '%s' account already exists and cannot be saved." % (self.formatted_name()))
 
         fields = {
@@ -773,6 +780,8 @@ class AdminAccount(object):
             if self.auth_plugin and self.auth_plugin != 'mysql_native_password':
                 if self.auth_string is None:
                     create_query = CREATE_USER_QUERY_PLUGIN
+                elif self.auth_plugin == 'caching_sha2_password':
+                    create_query = CREATE_USER_QUERY_PLUGIN_AUTH_CACHING
                 else:
                     create_query = CREATE_USER_QUERY_PLUGIN_AUTH_STRING
             else:
@@ -801,6 +810,7 @@ class AdminAccount(object):
         orig_revoked_privs =  all_normal_privs - self._orig_global_privs
         new_revoked_privs = all_normal_privs - self._global_privs - orig_revoked_privs
 
+
         if new_granted_privs or limits_changed:
             if 'Grant_priv' in new_granted_privs:
                 account_limits['GRANT'] = 'OPTION'
@@ -811,13 +821,22 @@ class AdminAccount(object):
                 priv_list = [ PrivilegeInfo[priv][0] for priv in new_granted_privs ]
             fields['granted_privs'] = ', '.join(priv_list) or 'USAGE'
             grant_query = GRANT_GLOBAL_PRIVILEGES_QUERY % fields
+
             with_clause = ''
             for key, value in account_limits.iteritems(): #name, value in zip(names, values):
                 if value != self._orig_account_limits.get(key):
+                    if key == "GRANT" and value == "OPTION":
+                        queries.append(grant_query + "WITH GRANT OPTION")
+                        continue
                     if not with_clause:
                         with_clause = ' WITH '
                     with_clause += "%s %s "%(key, value)
-            queries.append(grant_query + with_clause)
+                    
+            if self._owner.ctrl_be.target_version and self._owner.ctrl_be.target_version >= Version(8, 0, 5):
+                queries.append(grant_query)
+                queries.append((ALTER_USER_RESOURCES % fields) + with_clause)
+            else:
+                queries.append(grant_query + with_clause)
 
         if new_revoked_privs:
             if all_normal_privs - new_revoked_privs: #set(self._owner.global_privilege_names) - revoked_privs_set:  # Revoke a subset of all privs
@@ -831,10 +850,12 @@ class AdminAccount(object):
             change_pw = CHANGE_PASSWORD_QUERY if self._owner.ctrl_be.target_version and self._owner.ctrl_be.target_version < Version(5,7,6) else CHANGE_PASSWORD_QUERY_576
             blank_pw = BLANK_PASSWORD_QUERY if self._owner.ctrl_be.target_version and self._owner.ctrl_be.target_version < Version(5,7,6) else BLANK_PASSWORD_QUERY
             # special hack required by server to handle sha256 password accounts
-            if self.auth_plugin == "sha256_password":
-                queries.append("SET old_passwords = 2")
-            else:
-                queries.append("SET old_passwords = 0")
+            if self._owner.ctrl_be.target_version and self._owner.ctrl_be.target_version < Version(8, 0, 5):
+                if self.auth_plugin == "sha256_password":
+                    queries.append("SET old_passwords = 2")
+                else:
+                    queries.append("SET old_passwords = 0")
+                    
             if fields["password"]:
                 queries.append(change_pw % fields)
             else:
@@ -903,10 +924,10 @@ class AdminAccount(object):
         if not result.nextRow():
             raise Exception("Could not load account information for %s@%s"%(username,hostname))
 
-        self.username = result.stringByName("User")
-        self.host = result.stringByName("Host")
+        self.username = to_unicode(result.stringByName("User"))
+        self.host = to_unicode(result.stringByName("Host"))
 
-        self._orig_password = "UnchangedPassword\t"
+        self._orig_password = u"UnchangedPassword\t"
         self.password = self._orig_password
         self._orig_username = self.username
         self._orig_host = self.host
